"#import libs/utils.inc.src"
"#import libs/file.inc.src"
"#import libs/security.inc.src"

// Will clear the log file and all /Config/ and /.Trash/ folders as an additional security in case the server got misused before
// Can also accept a "proxy" flag to remove some unused binaries, to save a little bit of extra disc storage (accessible through FileExplorer)
// Follows the security guidelines at https://usine.solution-libre.fr/qtg/grey-hack/-/wikis/Securing-a-server, including the "gateway" proposal
// However, does NOT setup the permissions for "members" as such group is unsecure by design
// Author: laplongejr

// !!! This program will self-delete after use !!!
// Note : chmodding takes a bit of time for unknown reasons (and GH doesn't show prints while running)
// Expect ONE MINUTE or so until the single print statement shows up

//command: setup
if not imports.utils.IsRoot() then imports.utils.Exit("Must be root.")

length = imports.utils.ParamLength(params)
if length > 1 or imports.utils.HasFlag(params, "h", "help") then exit("<b>Usage: "+imports.utils.GetProgName()+" [(opt) number of users to add] --proxy</b>")
comp = get_shell.host_computer

ghostNbr = imports.utils.GetParam(params, 0)
if ghostNbr == null then ghostNbr = 0 else ghostNbr = ghostNbr.to_int
if ghostNbr < 0 then imports.utils.Exit("Can't add a negative amount of users, did you mean zero?")
isProxy = imports.utils.HasFlag(params, "p", "proxy")

// #0.1: Users able to bounce through ssh (ssh hosting missions)
SSH_GROUP = "ssh"
// #0.2: Users with /Public/Downloads/ (ftp hosting missions)
FTP_GROUP = "ftp"
// #0.3: Users connected to manipulate non-public files (subgroup)
MAIN_GROUP = "home"
// #0.4: Users with a private space (inherits home group)
USER_GROUP = "members"
// #0.6: (theorical) To not leak the root password during the handshake process
ADMIN_GROUP = "gateway"

// #1.1 By default, removing all perms for everyone on everything
main = comp.File("/")
for mod in ["u-rwx", "g-rwx", "o-rwx"]
	main.chmod(mod,1)
end for
// #1.2 Give read access to everyone in /
main.chmod("o+r",0)

// #1.3 Give execution rights to everyone for "/bin/clear"
for path in imports.file.SAFE
	file = comp.File(path)
	file.chmod("o+x",1)
end for
// #1.7 (...) /bin/sudo (...) and execution rights to everyone if there's no ssh!
if not isProxy then
    for path in imports.file.GATEWAY
    	file = comp.File(path)
    	if not file then continue
    	file.chmod("o+x",1)
    end for
end if

Safe = function(group, paths)
	for path in paths
		file = comp.File(path)
    	if not file then continue
		file.chmod("g+x",1)
		file.set_group(group,1)
	end for
end function
// #3.3 Set group for "/bin/ssh" as "ssh" and give it group-level execution rights
Safe(SSH_GROUP, imports.file.SSH)
 // Normally no-op
Safe(FTP_GROUP, imports.file.FTP)
// #1.7 Set group for /bin/sudo as "gateway" and give it group-level execute (...)
Safe(ADMIN_GROUP, imports.file.GATEWAY) // To allow the admin account to elevate as root without leaking in ssh

// #1.4 Create the groups and add them to users accordingly
// #2.1: Delete the home folder of (...) + "ssh"/"ftp"-only users
// Those tasks will be done with separate tools

// TODO: A bit sloppy, right?
chars=["a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"]
for i in range(0,nbrGhosts)
    name = chars[i]
    DEFAULT_PASS = "changeit"
    imports.security.NewLimitedUser(chars[i], DEFAULT_PASS, null, comp)
    print("User '"+name+"' create with password '"+DEFAULT_PASS+"'")
end for

SetRead = function(path, group)
	file = comp.File(path)
	if not file then return
	file.set_group(group,0)
	file.chmod("g+r",0)
end function
// #3.1 Set group for /home/ as "home" and give it group-level read
SetRead("/home/", MAIN_GROUP)
// #1.5 Set group for /Public/ as "ftp" and give it group-level read
SetRead("/Public/", FTP_GROUP)

// #1.6 Set group for /Public/Downloads/ as "ftp" and give it (recursive) group-level read
ftpDir = comp.File("/Public/Downloads/")
if ftpDir then
	ftpDir.set_group(FTP_GROUP,1)
	ftpDir.chmod("g+r", 1)
end if

// #2.1: Delete the home folder of guest + (...) users
imports.file.Delete(comp.File("/home/guest/"))
// #2.2: It would be a good idea to delete/reset the /Config/ and /.Trash/ folders for all users, just in case. (...)
imports.security.CleanUser("root", comp)

for user in comp.File("/home/").get_folders
	// #3.2 Give (recursive) !user-level! read for child folders of /home/
	user.chmod("u+r",1)
    // #2.2: It would be a good idea to delete/reset the /Config/ and /.Trash/ folders for all users, just in case. (...)
	imports.security.CleanUser(user.name, comp)
end for

// For allowing users to do the thing they are meant to do
// ... or saving some precious drive space ;)  
Optimize = function(useless, allowed)
    for path in useless
	    imports.security.WipeDir(path, comp)
    end for
    Safe(USER_GROUP, allowed)
end function

// #2.3 Nslookup, whois, Mail.exe aren't useful on a server, maaaybe on a dedicated proxy... delete them to free drive space! Same with CodeEditor.exe and build
// #3.4.2 FileExplorer.exe, Terminal.exe
Optimize(imports.file.USELESS_SHARED, imports.file.ALLOWED_SHARED)
if isProxy then
    // #2.4 Are members allowed the use this server as a last-point proxy to reach a ftp? If not, delete the ftp command
    Optimize(imports.file.USELESS_PROXY, imports.file.ALLOWED_PROXY)
else
    // #3.4.0 Set group "members" and give group-level execution rights for...
    // #3.4.1 cd, ls, pwd, cat (File access)
    Optimize(imports.file.USELESS_SERVICE, imports.file.ALLOWED_SERVICE)
end if

imports.utils.SelfDestruct()

// #2.2: It would be a good idea to delete/reset (...), just in case. and the log file too.
imports.file.Delete(comp.File("/var/system.log"))
comp.touch("/var/", "system.log")

print("Setup is finished")